#include <math.h>
#include <complex.h>


// **************************************************
// Formulas from Forshaw, Ross, White, hep-ph/0107232
// **************************************************

double THSM ( double Z, double s2w, double h );
double SHSM ( double Z, double h );

// **********************************************
// Formulas from He, Polonsky, Su, hep-ph/0102144
// **********************************************

double F ( double x1, double x2 );
double G ( double x );
double f ( double x1, double x2 );
double bet22 ( double qsq, double m1sq, double m2sq );
double bet0( double qsq, double m1sq, double m2sq );
double bbar0( double m1sq, double m2sq, double m3sq );
double SH_2HDM ( double Z,
		 double H, double A, double Hc, double h,
		 double b, double a );
double SH ( double Z, double h );
double TH_2HDM ( double Z, double W, double s2w,
		 double H, double A, double Hc, double h,
		 double b, double a );
double TH ( double Z, double W, double s2w, double h );

// IMPORTANT! He et.al. define hypercharge using Q=I_z+Y. For this reason Y=1/6 (-1/2) for quarks (leptons).
double SF ( double Y, double Nc, double m1, double m2, double Z );
double TF ( double Nc, double m1, double m2, double Z, double s2w );
double UF ( double Nc, double m1, double m2, double Z );

// ***********************************************************
// Majorana related formulae from Gates & Terning, PRL67, 1991
// ***********************************************************

double f1 ( double m1, double m2 );
double f2 ( double m1, double m2 );
double SM ( double m1, double m2, double mE );
double TM ( double m1, double m2, double mE, double s2w, double W );

// *******************************************************************
// Majorana related exact and approx formulae from Kniehl & Kohrs, PRD
// *******************************************************************

// Very small number to be used as imaginary part in complex integrals.
static const long double epsilon = 0.00005;
// PI() formulas have problems at q^2=0. We need a scale << m_Z^2.
static const long double little = 91.1875*91.1875*0.00004;

double A( double m1, double m2 );
double SMapprox( double m1, double m2, double mE );
double UMapprox( double m1, double m2, double mE );
double TMexact( double m1, double m2, double mE, double s2w, double W );
double complex lambda( double complex x, double y, double z );
long double complex B0(double s1, double m1sq, double m2sq );
long double complex PI( int vasign, double qsq, double m1, double m2 );
long double complex PI_V( double qsq, double m1, double m2 );
long double complex PI_A( double qsq, double m1, double m2 );
double SMexact( double m1, double m2, double mE, double s2w, double z );
double SpUMexact( double m1, double m2, double mE, double w, double z );

// **************************************
// Test methods for the Majorana formulae
// **************************************

long double testB0( double m );
void testSMvsSF( double m, double mE, double s2w, double Z );
double SMexactWithChecks( double m1, double m2, double mE, double s2w, double Z );
double UMexactWithChecks( double m1, double m2, double mE, double s2w, double Z, double smexact );

// ******************************
// Other functions of convenience
// ******************************

double SshiftFromRef( double h, double href, double t, double tref, double Z );
double TshiftFromRef( double h, double href, double t, double tref, double Z, double s2w );
